//use std::str::FromStr;

/// Ip address enum that includes associated type
/// 
/// # Example
/// ```
/// //Loopback Addresses:
/// IpAddr::V4(127, 0, 0, 1)
/// IpAddr::V6(String::from("::1"))
/// ```
#[allow(unused)]
pub enum IpAddr {
    V4(u8, u8, u8, u8),
    V6(String)
}

impl ToString for IpAddr {
    /// Function that returns an IP address in string form
    fn to_string(&self) -> String {
        match self {
            IpAddr::V4(oct1, oct2, oct3, oct4) => format!("{}.{}.{}.{}", oct1, oct2, oct3, oct4),
            IpAddr::V6(addr) => format!("{}", addr)
        }
    }
}

// impl FromStr for IpAddr {
//     /// Function that generates a IpAddr / Err from a string
//     fn from_str(s: &str) -> Result<Self, Self::Err> {
//         //Awating implementation
//     }
// }

/// Function that takes in an IP address and then prints a formatted string to the CLI
/// 
/// # Example
/// ```
/// let ip = IpAddr::V4(127, 0, 0, 1);
/// 
/// print_ip(ip);
/// //Output: IP Address: 127.0.0.1
/// ```
pub fn print_ip(ip_address: IpAddr) {
    println!("IP Address: {}", ip_address.to_string())
}

mod tests {
    #[test]
    /// Tests if an IP is converted to a string 
    /// correctly using the to_string trait
    fn ip_to_string() {
        use super::*;

        let ip = IpAddr::V4(192, 168, 0, 1);
        assert_eq!(ip.to_string(), "192.168.0.1")
    }
}